@@ -2,6 +2,7 @@
 #include "ClientSocket.hpp"
 #include "../../helpers/Log.hpp"
 #include "../../helpers/FFI.hpp"
+#include "../../Macros.hpp"
 #include "../message/MessageType.hpp"
 #include "../message/MessageParser.hpp"
 #include "../message/messages/GenericProtocolMessage.hpp"
@@ -16,6 +17,10 @@ CClientObject::CClientObject(SP<CClientSocket> client) : m_client(client) {
     ;
 }
 
+CClientObject::~CClientObject() {
+    TRACE(Debug::log(TRACE, "destroying object {}", m_id));
+}
+
 const std::vector<SMethod>& CClientObject::methodsOut() {
     return m_spec->c2s();
 }
@@ -25,11 +30,13 @@ const std::vector<SMethod>& CClientObject::methodsIn() {
 }
 
 void CClientObject::errd() {
-    m_client->m_error = true;
+    if (m_client)
+        m_client->m_error = true;
 }
 
 void CClientObject::sendMessage(SP<CGenericProtocolMessage> msg) {
-    m_client->sendMessage(msg);
+    if (m_client)
+        m_client->sendMessage(msg);
 }
 
 SP<IServerClient> CClientObject::client() {
