@@ -2,6 +2,7 @@
 #include "ServerClient.hpp"
 #include "ServerSocket.hpp"
 #include "../../helpers/Log.hpp"
+#include "../../Macros.hpp"
 #include "../message/MessageType.hpp"
 #include "../message/MessageParser.hpp"
 #include "../message/messages/GenericProtocolMessage.hpp"
@@ -17,6 +18,10 @@ CServerObject::CServerObject(SP<CServerClient> client) : m_client(client) {
     ;
 }
 
+CServerObject::~CServerObject() {
+    TRACE(Debug::log(TRACE, "[{}] destroying object {}", m_client->m_fd.get(), m_id));
+}
+
 const std::vector<SMethod>& CServerObject::methodsOut() {
     return m_spec->s2c();
 }
@@ -26,11 +31,13 @@ const std::vector<SMethod>& CServerObject::methodsIn() {
 }
 
 void CServerObject::errd() {
-    m_client->m_error = true;
+    if (m_client)
+        m_client->m_error = true;
 }
 
 void CServerObject::sendMessage(SP<CGenericProtocolMessage> msg) {
-    m_client->sendMessage(msg);
+    if (m_client)
+        m_client->sendMessage(msg);
 }
 
 SP<IServerClient> CServerObject::client() {
